// RTOS_2 2nd Example of using FreeRTOS
// by Joe Jaworski

#include <Arduino_FreeRTOS.h>
#include <semphr.h>

const uint8_t LEDPin = 13;                      // Built-in LED I/O Pin

enum uint_8 {
    LED_OFF,                                    // LED ON
    LED_ON,                                     // LED Off
    LED_Blink                                   // Blinking LED
};

volatile SemaphoreHandle_t SemaStatus;         // LED status semaphore
volatile uint8_t g_LEDStatus = LED_Blink;      // Current LED status

#define rtDelay(v) {vTaskDelay(v/15);}         // delay in milliseconds



// ***************
// setup() function
//
void setup()
{

    SemaStatus = xSemaphoreCreateBinary();      // Create semaphore
    xSemaphoreGive(SemaStatus);                 // must GIVE after creation


// Setup our LED Blinky Task
    xTaskCreate
    (
        TaskBlink,                      // function name
        (const portCHAR *)"blink",      // task name
        128,                            // stack size
        NULL,                           // No creation parms
        2,                              // Priority
        NULL                            // returned task handle
    );


// Setup our Serial Comm Task
    xTaskCreate
    (
        TaskComm,                       // function name
        (const portCHAR *)"comm",       // task name
        128,                            // stack size
        NULL,                           // No creation parms
        2,                              // Priority
        NULL                            // returned task handle
    );


} // End Setup




// *****************
// loop() function
//
void loop()
{
    /* Nothing to do here! */

} // End loop()




// **********************
// TaskBlink() function
//
void TaskBlink(void*)
{
    uint8_t Status;

    pinMode(LEDPin, OUTPUT);            // Set LED I/O pin to OUTPUT

    for(;;)
    {
        xSemaphoreTake(SemaStatus, portMAX_DELAY);
        Status = g_LEDStatus;                      // get a copy
        xSemaphoreGive(SemaStatus);

        switch(Status)
        {
            case LED_OFF:
            digitalWrite(LEDPin, LOW);             // Turn off Led
            break;

            case LED_ON:
            digitalWrite(LEDPin, HIGH);            // Turn on Led
            break;

            case LED_Blink:
            if( digitalRead(LEDPin) )
                 digitalWrite(LEDPin, LOW);         // toggle Led state
            else
                 digitalWrite(LEDPin, HIGH);
            break;

            default:
            break;
        }

        rtDelay(500);

    }

} // End TaskBlink



// **********************
// TaskComm() function
//
void TaskComm(void*)
{
    uint8_t ByteRead = 0;

    Serial.begin(9600);                             // Open Serial Monitor

    for(;;)
    {
       while(Serial.available() > 0)
       {
          (void)Serial.read();                      // flush serial buffer
       }
        Serial.println("\r\nSelect LED state:");
        Serial.println("1) LED Off");
        Serial.println("2) LED On");
        Serial.println("3) LED Blink");
        Serial.println("Enter 1, 2, or 3: ");

        // Read in the users selection
        while( Serial.available() == 0 )
        {
            rtDelay(60);
        }
        ByteRead = Serial.read();               // read the incoming byte

        if( (ByteRead != '1') && (ByteRead != '2') &&  (ByteRead != '3') )
        {
            Serial.print("\r\nError: Please enter 1, 2, or 3");
        }

        xSemaphoreTake(SemaStatus, portMAX_DELAY); // block other tasks

        switch(ByteRead)
        {
            case '1': g_LEDStatus = LED_OFF;   break;
            case '2': g_LEDStatus = LED_ON;    break;
            case '3': g_LEDStatus = LED_Blink; break;
            default: break;
        }
        xSemaphoreGive(SemaStatus);

    }

} // End TaskComm
